/*
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this code; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  */

/* Copyright (C) 2000 Ian Zimmerman <itz@speakeasy.org> */

static char rcsid[] = "$Id: sockets.c,v 1.18 2002/06/04 06:54:11 itz Exp $";

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/socket.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <paths.h>
#include <syslog.h>
#include <sys/un.h>
#include <unistd.h>
#include <assert.h>
#include <stdlib.h>

#include "sockets.h"
#include "connections.h"
#include "preadwrite.h"
#include "gim_common.h"
#include "exc.h"

int
open_unix (const char* path)
{
  int fd;
  struct sockaddr_un addr;
  struct sockaddr* paddr;

  fd = -1;
  exc_try {
    if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
      int my_errno = errno;
      syslog(LOG_ERR, "socket(AF_UNIX) failed: %m");
      exc_eraise2(unix_error, my_errno);
    } /*if*/

    memset(&addr, 0, sizeof(addr));
    addr.sun_family = AF_UNIX;
    strcpy(addr.sun_path, path);

    paddr = (struct sockaddr*)&addr;
    if (bind(fd, paddr, sizeof(struct sockaddr_un)) == -1) {
      int my_errno = errno;
      syslog(LOG_ERR, "bind(%s) failed: %m, is another daemon running?", path);
      exc_eraise2(unix_error, my_errno);
    } /*if*/
  
    if (listen(fd, 5) == -1) {
      int my_errno = errno;
      syslog(LOG_ERR, "listen() failed: %m");
      exc_eraise2(unix_error, my_errno);
    } /*if*/
  }
  exc_except(exc_any) {
    if (fd != -1) close(fd);
    exc_ereraise;
  } exc_end;

  return fd;
}

int
add_client (int fd, int* max_client, fd_set* all_client_set)
{
  char blurb[100];
  int len;
  int s;

  if ((s = accept(fd, 0, 0)) == -1) {
    int my_errno = errno;
    syslog(LOG_ERR, "accept() failed: %m");
    exc_eraise2(unix_error, my_errno);
  } /*if*/
  
  FD_SET(s, all_client_set);
  if (s + 1 > *max_client) {
    *max_client = s + 1;
  } /*if*/

  len = snprintf(blurb, sizeof(blurb), gim_greeting_format, PROTOCOL);
  assert(len >= 0 && len < sizeof(blurb));
  gim_pwrite(s, blurb, len);
  return s;
}

void
delete_client (int fd, int *max_client, fd_set* all_client_set)
{
  int i;

  FD_CLR(fd, all_client_set);
  close(fd);

  if (fd + 1 == *max_client) {
    for (i = *max_client - 1; i >= 0; i = i - 1) {
      if (FD_ISSET(i, all_client_set)) {
        break;
      } /*if*/
    } /*for*/
    *max_client = i + 1;
  } /*if*/

}

void
authenticate_client (int fd, struct ucred* psucr)
{
  socklen_t credlen;
  char* uname;

  credlen = sizeof(struct ucred);  
  if (-1 == getsockopt(fd, SOL_SOCKET, SO_PEERCRED, psucr, &credlen)) {
    int my_errno = errno;
    syslog(LOG_ERR, "getsockopt(SO_PEERCRED) failed: %m");
    exc_eraise2(unix_error, my_errno);
  } /*if*/

}
